"""
    lyricy
    ~~~~~~~~~~
    A command line lyrics utility tool which search and add lyrics to your offline songs.
"""

__package__ = "lyricy"
__description__ = (
    "A command line lyrics utility tool which search and add lyrics to your offline songs"
)
__url__ = "https://github.com/yogeshwaran01/lyricy"
__version__ = "1.2"
__author__ = "YOGESHWARAN R <yogeshin247@gmail.com>"
__license__ = "MIT License"
__copyright__ = "Copyright 2022 Yogeshwaran R"


from enum import Enum

import music_tag

from .classes import BaseLyrics
from .cli import lyrics_without_tags
from .providers import Megalobiz, RcLyricsBand


class Providers(Enum):
    MEGALOBIZ = Megalobiz
    RCLYRICSBAND = RcLyricsBand


class Lyrics(BaseLyrics):
    def fetch(self):
        """Fetch the full lyrics of the song with lrc tags"""
        if "rclyricsband" in self.link:
            self.lyrics = RcLyricsBand.get_lyrics(self.link)
        else:
            self.lyrics = Megalobiz.get_lyrics(self.link)
        self.lyrics_without_lrc_tags = lyrics_without_tags(self.lyrics)

    def save(self, path: str):
        """Save the lyrics file"""
        with open(path, "w") as file:
            file.write(self.lyrics)

    def add_to_track(self, path: str):
        """
        Add the lyrics to track metadata
        `path`: path of the track
        """
        f = music_tag.load_file(path)
        f["lyrics"] = self.lyrics
        f.save()


class Lyricy:
    @staticmethod
    def search(query: str, provider=Providers.MEGALOBIZ) -> list[Lyrics]:
        """Search for a lyrics for given Query"""
        if provider == Providers.RCLYRICSBAND:
            r = RcLyricsBand.search_lyrics(query)
            return [Lyrics(**i.__dict__) for i in r]

        r = Megalobiz.search_lyrics(query)
        return [Lyrics(**i.__dict__) for i in r]



import sys

import click
import music_tag
import pylrc
from rich import print
from rich.columns import Columns
from rich.console import Console
from rich.panel import Panel

from .classes import BaseLyrics
from .providers import Megalobiz, RcLyricsBand

console = Console()


def format_table(lyrics: BaseLyrics, disable_preview: bool) -> str:
    title = lyrics.title
    sample_lyrics = lyrics.sample_lyrics
    index = lyrics.index
    if disable_preview:
        return f"[b]({index}){title}[/b]"
    return f"[b]({index}){title}[/b]\n[yellow]{sample_lyrics}"


def print_help_msg(command: callable):
    with click.Context(command) as ctx:
        click.echo(command.get_help(ctx))


def lyrics_without_tags(lyrics_with_lrc_tags: str) -> str:
    """Show Lyrics Only (without LRC tag)"""
    parsed_text = pylrc.parse(lyrics_with_lrc_tags)
    return "\n".join([line.text for line in parsed_text])


@click.group()
def cli():
    """
    A command line lyrics utility tool which search and add lyrics to your offline songs.
    """
    pass


@click.command()
@click.option("-t", "--track", type=click.Path(exists=True), help="file path of track")
@click.option("--disable-preview", "-d", is_flag=True, help="Disable the preview")
@click.option(
    "--only-lyrics", "-l", is_flag=True, help="Show Lyrics Only (without LRC tag)"
)
@click.option("-s", "--save", help="Save file as .lrc")
@click.option("-q", "--query", type=str, help="search query of track name")
@click.option("-p", "--provider", type=str, help="Lyrics provider name [rc] or [mo]")
def search(track: str, query: str, disable_preview: bool, only_lyrics: bool, save: str, provider: str):
    """Search for lyrics for given track or query"""
    if track:
        f = music_tag.load_file(track)
        title = str(f["title"])
    elif query:
        title = query

    else:
        print_help_msg(search)
        sys.exit()

    with console.status(f"[bold green]Searching lyrics for {title}") as _:
        if provider == "rc":
            results = RcLyricsBand.search_lyrics(title)
        else:
            results = Megalobiz.search_lyrics(title)

    songs_lyrics_renderables = [
        Panel(format_table(result, disable_preview), expand=True) for result in results
    ]
    console.print(Columns(songs_lyrics_renderables))

    selected_lyrics_index = str(
        click.prompt("Enter the index of lyrics", type=int, default=1) - 1
    )

    selected_lyrics = results[int(selected_lyrics_index)]

    with console.status("[bold green]Fetching Lyrics") as _:
        if provider == "rc":
            lyric = RcLyricsBand.get_lyrics(selected_lyrics.link)
        else:
            lyric = Megalobiz.get_lyrics(selected_lyrics.link)

    if only_lyrics:
        if save:
            with open(f"{save}.lrc", "w") as file:
                file.write(lyrics_without_tags(lyric))
        print(lyrics_without_tags(lyric))
    else:
        if save:
            with open(f"{save}.lrc", "w") as file:
                file.write(lyric)
        print(lyric)


@click.command()
@click.argument("track", type=click.Path(exists=True))
@click.option("--query", "-q", help="search for this query instead of track name")
@click.option("--disable-preview", "-d", is_flag=True, help="Disable the preview")
@click.option("--show", is_flag=True, help="Print the lyrics and ask for confirmation")
@click.option("--lrc", type=click.Path(exists=True), help="Lyrics file to add on track")
@click.option("-p", "--provider", type=str, help="Lyrics provider name [rc] or [mo]")
def add(track: str, show: bool, disable_preview: bool, lrc: str, query: str, provider: str):
    """Search and add lyrics to given TRACK.

    TRACK is the filepath of track.
    """
    f = music_tag.load_file(track)
    if lrc:
        with open(lrc, "r") as file:
            lyric = file.read()
    else:
        if query:
            title = query
        else:
            title = str(f["title"])
        with console.status(f"[bold green]Searching lyrics for {title}") as _:
            if provider == "rc":
                results = RcLyricsBand.search_lyrics(title)
            else:
                results = Megalobiz.search_lyrics(title)
        songs_lyrics_renderables = [
            Panel(format_table(result, disable_preview), expand=True)
            for result in results
        ]
        console.print(Columns(songs_lyrics_renderables))

        selected_lyrics_index = str(
            click.prompt("Enter the index of lyrics", type=int, default=1) - 1
        )

        selected_lyrics = results[int(selected_lyrics_index)]
        with console.status("[bold green]Fetching Lyrics") as _:
            if provider == "rc":
                lyric = RcLyricsBand.get_lyrics(selected_lyrics.link)
            else:
                lyric = Megalobiz.get_lyrics(selected_lyrics.link)

    if show:
        print(lyric)

        if click.confirm("Do you want add this lyrics?", abort=True):
            with console.status("[bold green]Adding Lyrics") as _:
                f["lyrics"] = lyric
                f.save()
    with console.status("[bold green]Adding Lyrics") as _:
        f["lyrics"] = lyric
        f.save()
        click.echo("✨ Done ✨")


@click.command()
@click.argument("track", type=click.Path(exists=True))
def remove(track):
    """Remove lyrics from given TRACK.

    TRACK is the filepath of track.
    """
    f = music_tag.load_file(track)
    f["lyrics"] = ""
    f.save()
    click.echo("✨ Done ✨")


@click.command()
@click.argument("track", type=click.Path(exists=True))
@click.option(
    "--only-lyrics", "-l", is_flag=True, help="Show Lyrics Only (without LRC tag)"
)
def show(track, only_lyrics):
    """Show the lyrics of TRACK if available.

    TRACK is the filepath of track.
    """
    f = music_tag.load_file(track)
    lyric = str(f["lyrics"])
    if only_lyrics:
        print(lyrics_without_tags(lyric))
    else:
        print(lyric)


cli.add_command(search)
cli.add_command(add)
cli.add_command(remove)
cli.add_command(show)



from lyricy import Lyricy, Lyrics, Providers

TEST_QUERY_1 = "karka karka"
FILE_PATH_1 = "karka_karka.lrc"

TEST_QUERY_2 = "beast mode"
FILE_PATH_2 = "beast_mode.lrc"


def test_lyricy_megalobiz():
    lyricy = Lyricy()
    results: list[Lyrics] = lyricy.search(TEST_QUERY_1)
    selected_lyrics = results[0]
    selected_lyrics.fetch()
    assert type(selected_lyrics.lyrics) == str
    assert type(selected_lyrics.lyrics_without_lrc_tags) == str
    selected_lyrics.save(FILE_PATH_1)

    with open(FILE_PATH_1) as file:
        text = file.read()

    assert text == selected_lyrics.lyrics


def test_lyricy_rclyricsband():
    lyricy = Lyricy()
    results: list[Lyrics] = lyricy.search(TEST_QUERY_2, provider=Providers.RCLYRICSBAND)
    selected_lyrics = results[0]
    selected_lyrics.fetch()
    assert type(selected_lyrics.lyrics) == str
    assert type(selected_lyrics.lyrics_without_lrc_tags) == str
    selected_lyrics.save(FILE_PATH_2)

    with open(FILE_PATH_2) as file:
        text = file.read()

    assert text == selected_lyrics.lyrics



from urllib.parse import quote_plus

import requests
from bs4 import BeautifulSoup

from .classes import BaseLyrics


class Megalobiz:
    """Search and scrape lyrics for Megalobiz site"""

    def search_lyrics(song_name: str) -> list[BaseLyrics]:
        """Search for lyrics"""

        results = []
        search_link = "https://www.megalobiz.com/search/all?qry="
        markup = requests.get(search_link + quote_plus(song_name)).text
        soup = BeautifulSoup(markup, "html.parser")
        required_tags = soup.find_all("a", {"class": "entity_name"})

        outer_tags = soup.findAll("div", {"class": "details"})

        inner_tags = [
            outer_tags[i].find_all("span")[-1] for i in range(0, len(outer_tags), 2)
        ]

        sample_lyrics_list = [i.text for i in inner_tags]

        for index, tag in enumerate(required_tags):
            results.append(
                BaseLyrics(
                    title=tag.get("title"),
                    link="https://www.megalobiz.com" + tag.get("href"),
                    sample_lyrics=sample_lyrics_list[index],
                    index=str(index + 1),
                )
            )

        if len(results) == 0:
            return [
                BaseLyrics(
                    title=" No result found", link="", sample_lyrics="", index="1"
                )
            ]

        else:
            return results

    def get_lyrics(link: str) -> str:
        """Scrape the lyrics for given track link"""

        markup = requests.get(link).text
        soup = BeautifulSoup(markup, "html.parser")
        return (
            soup.find("div", {"class": "lyrics_details entity_more_info"})
            .find("span")
            .text
        )


class RcLyricsBand:
    """Search and scrape lyrics for RcLyricsBand site"""

    def search_lyrics(song_name: str):
        """Search for lyrics"""

        search_link = "https://rclyricsband.com/?s="
        markup = requests.get(search_link + quote_plus(song_name)).text
        soup = BeautifulSoup(markup, "html.parser")
        outer_tags = soup.find_all("h2", {"class": "search-entry-title"})
        results = []
        for index, outer_tag in enumerate(outer_tags):
            inner_tag = outer_tag.find("a")
            results.append(
                BaseLyrics(
                    title=inner_tag.get("title"),
                    link=inner_tag.get("href"),
                    sample_lyrics="",
                    index=str(index + 1),
                )
            )
        if len(results) == 0:
            return [
                BaseLyrics(
                    title=" No result found", link="", sample_lyrics="", index="1"
                )
            ]
        else:
            return results

    def get_lyrics(link: str):
        """Scrape the lyrics for given track link"""

        markup = requests.get(link).text
        soup = BeautifulSoup(markup, "html.parser")
        return soup.find("div", {"class": "su-box su-box-style-default"}).text



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

setup(
    name='lyricy',
    version='1.2',
    author='Yogeshwaran R',
    author_email='yogeshin247@gmail.com',
    description='A command line lyrics utitly tool which \
    search and add lyrics to your offline songs.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    url='https://github.com/yogeshwaran01/lyricy',
    download_url="https://github.com/yogeshwaran01/lyricy/archive/master.zip",
    packages=find_packages(),
    entry_points={
            'console_scripts': [
                'lyricy=lyricy.cli:cli'
            ]
    },
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    keywords='python package lyrics lrc yogeshwaran01 songs',
    install_requires=requirements,

)
